package com.chinaweal.aiccs.aiccs.abnormal.mapper;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaweal.aiccs.aiccs.abnormal.entity.AbnStatistics;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.chinaweal.aiccs.aiccs.abnormal.entity.dto.AbnStatDto;
import com.chinaweal.aiccs.aiccs.abnormal.entity.dto.AbnStatisticsDto;
import com.chinaweal.aiccs.aiccs.abnormal.entity.excel.StatisticsAbnExcelDto;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 经营异常目录统计表 Mapper 接口
 * </p>
 *
 * @author sh
 * @since 2021-12-21
 */
public interface AbnStatisticsMapper extends BaseMapper<AbnStatistics> {

    AbnStatistics queryStatByDate(@Param("xzqh") String xzqh, @Param("abnTime") String abnTime, @Param("entType") String entType);

    List<AbnStatisticsDto> queryStatAll(@Param("customParamMap") Map<String, Object> customParamMap);

    AbnStatDto statisticsForAbn(@Param("customParamMap") Map<String, Object> customParamMap);

    AbnStatDto statisticsForAbnHz(@Param("customParamMap") Map<String, Object> customParamMap);

    // 查区局
    List<AbnStatDto> statisticsForAbnQJ(@Param("customParamMap") Map<String, Object> customParamMap);

    IPage<StatisticsAbnExcelDto> getEntListByStatistics(Page<StatisticsAbnExcelDto> page, @Param("customParamMap") Map<String, Object> customParamMap);
}
